@@ -2,7 +2,8 @@
 #[structopt(
         setting = structopt::clap::AppSettings::UnifiedHelpMessage,
         setting = structopt::clap::AppSettings::DeriveDisplayOrder,
-        setting = structopt::clap::AppSettings::DontCollapseArgsInUsage
+        setting = structopt::clap::AppSettings::DontCollapseArgsInUsage,
+        setting = concolor_clap::color_choice(),
     )]
 #[structopt(group = structopt::clap::ArgGroup::with_name("mode").multiple(false))]
 pub struct Args {
@@ -58,7 +59,7 @@ pub struct Args {
     pub dump_config: Option<std::path::PathBuf>,
 
     #[structopt(flatten)]
-    pub color: git_stack::color::ColorArgs,
+    pub(crate) color: concolor_clap::Color,
 
     #[structopt(flatten)]
     pub verbose: clap_verbosity_flag::Verbosity,
