@@ -85,7 +85,7 @@ impl State {
                 }]
             }
             (None, None, git_stack::config::Stack::All) => {
-                let mut stack_branches = std::collections::HashMap::new();
+                let mut stack_branches = std::collections::BTreeMap::new();
                 for (branch_id, branch) in branches.iter() {
                     let base_branch =
                         resolve_implicit_base(&repo, branch_id, &branches, &protected_branches)
@@ -193,6 +193,17 @@ impl StackState {
         self.branches.update(repo);
         Ok(())
     }
+
+    fn graphed_branches(&self) -> git_stack::git::Branches {
+        let mut graphed_branches = self.branches.clone();
+        if !graphed_branches.contains_oid(self.base.id) {
+            graphed_branches.insert(self.base.clone());
+        }
+        if !graphed_branches.contains_oid(self.onto.id) {
+            graphed_branches.insert(self.onto.clone());
+        }
+        graphed_branches
+    }
 }
 
 pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitResult {
@@ -207,6 +218,14 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
             return Err(proc_exit::Code::USAGE_ERR.with_message("Working tree is dirty, aborting"));
         }
 
+        // Update status of remote unprotected branches
+        match git_fetch(&mut state.repo) {
+            Ok(_) => (),
+            Err(err) => {
+                log::warn!("Skipping fetch of `{}`, {}", state.repo.push_remote(), err);
+            }
+        }
+
         let mut pulled_ids = HashSet::new();
         for stack in state.stacks.iter() {
             let mut stack_pulled_ids = HashSet::new();
@@ -255,6 +274,7 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
 
     const BACKUP_NAME: &str = "git-stack";
     let mut success = true;
+    let mut backed_up = false;
     if state.rebase {
         if state.repo.is_dirty() {
             return Err(proc_exit::Code::USAGE_ERR.with_message("Working tree is dirty, aborting"));
@@ -265,7 +285,10 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
         let mut backup = git_stack::backup::Backup::from_repo(&state.repo)
             .with_code(proc_exit::Code::FAILURE)?;
         backup.insert_parent(&state.repo, &state.branches, &state.protected_branches);
-        backups.push(backup)?;
+        if !state.dry_run {
+            backups.push(backup)?;
+            backed_up = true;
+        }
 
         let head_branch = state
             .repo
@@ -304,8 +327,8 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
 
     show(&state, colored_stdout).with_code(proc_exit::Code::FAILURE)?;
 
-    if state.rebase {
-        log::info!("To undo, run `git branch-backup pop {}", BACKUP_NAME);
+    if backed_up {
+        log::info!("To undo, run `git branch-backup pop {}`", BACKUP_NAME);
     }
 
     if !success {
@@ -316,21 +339,12 @@ pub fn stack(args: &crate::args::Args, colored_stdout: bool) -> proc_exit::ExitR
 }
 
 fn plan_rebase(state: &State, stack: &StackState) -> eyre::Result<git_stack::git::Script> {
-    let mut graphed_branches = stack.branches.clone();
-    if !graphed_branches.contains_oid(stack.base.id) {
-        graphed_branches.insert(stack.base.clone());
-    }
-    if !graphed_branches.contains_oid(stack.onto.id) {
-        graphed_branches.insert(stack.onto.clone());
-    }
+    let mut graphed_branches = stack.graphed_branches();
     let mut root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-    root = root.extend(&state.repo, graphed_branches)?;
+    root = root.extend_branches(&state.repo, graphed_branches)?;
+    git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
 
-    git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches)?;
-
-    git_stack::graph::rebase_branches(&mut root, stack.onto.id)?;
-
-    git_stack::graph::delinearize(&mut root);
+    git_stack::graph::rebase_branches(&mut root, stack.onto.id);
 
     let script = git_stack::graph::to_script(&root);
 
@@ -340,21 +354,14 @@ fn plan_rebase(state: &State, stack: &StackState) -> eyre::Result<git_stack::git
 fn push(state: &mut State) -> eyre::Result<()> {
     let mut graphed_branches = git_stack::git::Branches::new(None.into_iter());
     for stack in state.stacks.iter() {
-        graphed_branches.extend(stack.branches.iter().flat_map(|(_, b)| b.to_owned()));
-    }
-    for stack in state.stacks.iter() {
-        if !graphed_branches.contains_oid(stack.base.id) {
-            graphed_branches.insert(stack.base.clone());
-        }
-        if !graphed_branches.contains_oid(stack.onto.id) {
-            graphed_branches.insert(stack.onto.clone());
-        }
+        let stack_graphed_branches = stack.graphed_branches();
+        graphed_branches.extend(stack_graphed_branches.into_iter().flat_map(|(_, b)| b));
     }
     let mut root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-    root = root.extend(&state.repo, graphed_branches)?;
+    root = root.extend_branches(&state.repo, graphed_branches)?;
 
-    git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches)?;
-    git_stack::graph::pushable(&mut root)?;
+    git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
+    git_stack::graph::pushable(&mut root);
 
     git_push(&mut state.repo, &root, state.dry_run)?;
 
@@ -362,27 +369,34 @@ fn push(state: &mut State) -> eyre::Result<()> {
 }
 
 fn show(state: &State, colored_stdout: bool) -> eyre::Result<()> {
-    let mut graphed_branches = git_stack::git::Branches::new(None.into_iter());
-    for stack in state.stacks.iter() {
-        graphed_branches.extend(stack.branches.iter().flat_map(|(_, b)| b.to_owned()));
-    }
-    for stack in state.stacks.iter() {
-        if !graphed_branches.contains_oid(stack.base.id) {
-            graphed_branches.insert(stack.base.clone());
-        }
-        if !graphed_branches.contains_oid(stack.onto.id) {
-            graphed_branches.insert(stack.onto.clone());
-        }
-    }
-    let mut root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
-    root = root.extend(&state.repo, graphed_branches)?;
+    let mut roots = state
+        .stacks
+        .iter()
+        .map(|stack| -> eyre::Result<git_stack::graph::Node> {
+            let mut graphed_branches = stack.graphed_branches();
+            let mut root =
+                git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
+            root = root.extend_branches(&state.repo, graphed_branches)?;
+            git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches);
 
-    git_stack::graph::protect_branches(&mut root, &state.repo, &state.protected_branches)?;
-    git_stack::graph::pushable(&mut root)?;
-    if !state.show_stacked {
-        git_stack::graph::delinearize(&mut root);
+            if state.dry_run {
+                // Show as-if we performed all mutations
+                git_stack::graph::rebase_branches(&mut root, stack.onto.id);
+            }
+
+            eyre::Result::Ok(root)
+        });
+    let mut root = roots.next().unwrap_or_else(|| {
+        let mut graphed_branches = git_stack::git::Branches::new(None.into_iter());
+        let root = git_stack::graph::Node::new(state.head_commit.clone(), &mut graphed_branches);
+        Ok(root)
+    })?;
+    for other in roots {
+        root = root.extend(&state.repo, other?)?;
     }
 
+    git_stack::graph::pushable(&mut root);
+
     match state.show_format {
         git_stack::config::Format::Silent => (),
         git_stack::config::Format::Branches
@@ -394,6 +408,8 @@ fn show(state: &State, colored_stdout: bool) -> eyre::Result<()> {
                 DisplayTree::new(&state.repo, &root)
                     .colored(colored_stdout)
                     .show(state.show_format)
+                    .stacked(state.show_stacked)
+                    .protected_branches(&state.protected_branches)
             )?;
         }
         git_stack::config::Format::Debug => {
@@ -438,6 +454,23 @@ fn resolve_implicit_base(
     Ok(branch.clone())
 }
 
+fn git_fetch(repo: &mut git_stack::git::GitRepo) -> eyre::Result<()> {
+    let remote = repo.push_remote();
+    log::debug!("git fetch {}", remote);
+    // A little uncertain about some of the weirder authentication needs, just deferring to `git`
+    // instead of using `libgit2`
+    let status = std::process::Command::new("git")
+        .arg("fetch")
+        .arg(remote)
+        .status()
+        .wrap_err("Could not run `git fetch`")?;
+    if !status.success() {
+        eyre::bail!("`git fetch {}` failed", remote);
+    }
+
+    Ok(())
+}
+
 fn git_pull(
     repo: &mut git_stack::git::GitRepo,
     branch_name: &str,
@@ -714,7 +747,7 @@ fn git_push_internal(
                     }
                 }
             }
-        } else if node.action.is_protected() || node.action.is_rebase() {
+        } else if node.action.is_protected() {
             log::debug!("Skipping push of `{}`, protected", branch.name);
         } else {
             log::debug!("Skipping push of `{}`", branch.name);
@@ -722,30 +755,32 @@ fn git_push_internal(
     }
 
     if failed.is_empty() {
-        for stack in node.stacks.iter() {
-            for stack_node in stack.iter() {
-                failed.extend(git_push_internal(repo, stack_node, dry_run));
-            }
+        for child in node.children.values() {
+            failed.extend(git_push_internal(repo, child, dry_run));
         }
     }
 
     failed
 }
 
-struct DisplayTree<'r, 'n> {
+struct DisplayTree<'r> {
     repo: &'r git_stack::git::GitRepo,
-    root: &'n git_stack::graph::Node,
+    root: &'r git_stack::graph::Node,
+    protected_branches: git_stack::git::Branches,
     palette: Palette,
     show: git_stack::config::Format,
+    stacked: bool,
 }
 
-impl<'r, 'n> DisplayTree<'r, 'n> {
-    pub fn new(repo: &'r git_stack::git::GitRepo, root: &'n git_stack::graph::Node) -> Self {
+impl<'r> DisplayTree<'r> {
+    pub fn new(repo: &'r git_stack::git::GitRepo, root: &'r git_stack::graph::Node) -> Self {
         Self {
             repo,
             root,
+            protected_branches: Default::default(),
             palette: Palette::plain(),
             show: Default::default(),
+            stacked: Default::default(),
         }
     }
 
@@ -762,79 +797,218 @@ impl<'r, 'n> DisplayTree<'r, 'n> {
         self.show = show;
         self
     }
+
+    pub fn stacked(mut self, stacked: bool) -> Self {
+        self.stacked = stacked;
+        self
+    }
+
+    pub fn protected_branches(mut self, protected_branches: &git_stack::git::Branches) -> Self {
+        self.protected_branches = protected_branches.clone();
+        self
+    }
 }
 
-impl<'r, 'n> std::fmt::Display for DisplayTree<'r, 'n> {
+impl<'r> std::fmt::Display for DisplayTree<'r> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
-        let mut tree = treeline::Tree::root(RenderNode {
-            repo: self.repo,
-            node: Some(self.root),
-            palette: &self.palette,
-        });
-        to_tree(
+        let head_branch = self.repo.head_branch().unwrap();
+        let mut tree = to_tree(
             self.repo,
-            self.root.stacks.as_slice(),
-            &mut tree,
+            &head_branch,
+            &self.protected_branches,
+            self.root,
             &self.palette,
-            self.show,
         );
+        if self.stacked {
+            tree.linearize();
+        } else {
+            tree.sort();
+        }
+        match self.show {
+            git_stack::config::Format::Silent => tree.skip(|_| true),
+            git_stack::config::Format::Commits => tree.skip(|_| false),
+            git_stack::config::Format::BranchCommits => tree.skip(|tree| {
+                if let Some(node) = tree.root.node {
+                    let protected = node.action.is_protected();
+                    let boring_commit = node.branches.is_empty() && tree.stacks.is_empty();
+                    protected && boring_commit
+                } else {
+                    false
+                }
+            }),
+            git_stack::config::Format::Branches => tree.skip(|tree| {
+                if let Some(node) = tree.root.node {
+                    let boring_commit = node.branches.is_empty() && tree.stacks.is_empty();
+                    boring_commit
+                } else {
+                    false
+                }
+            }),
+            git_stack::config::Format::Debug => tree.skip(|_| false),
+        }
+        let tree = tree.into_display();
         tree.fmt(f)
     }
 }
 
-fn to_tree<'r, 'n, 'p>(
+fn to_tree<'r>(
     repo: &'r git_stack::git::GitRepo,
-    nodes: &'n [Vec<git_stack::graph::Node>],
-    tree: &mut treeline::Tree<RenderNode<'r, 'n, 'p>>,
-    palette: &'p Palette,
-    show: git_stack::config::Format,
-) {
-    for branch in nodes {
-        let mut branch_root = treeline::Tree::root(RenderNode {
+    head_branch: &'r git_stack::git::Branch,
+    protected_branches: &'r git_stack::git::Branches,
+    node: &'r git_stack::graph::Node,
+    palette: &'r Palette,
+) -> Tree<'r> {
+    let mut weight = if node.action.is_protected() {
+        Weight::Protected(0)
+    } else if node.local_commit.id == head_branch.id {
+        Weight::Head(0)
+    } else {
+        Weight::Commit(0)
+    };
+
+    let mut stacks = Vec::new();
+    for child in node.children.values() {
+        let child_tree = to_tree(repo, head_branch, protected_branches, child, palette);
+        weight = weight.max(child_tree.weight);
+        stacks.push(vec![child_tree]);
+    }
+
+    let tree = Tree {
+        root: RenderNode {
             repo,
-            node: None,
+            head_branch,
+            protected_branches,
+            node: Some(node),
             palette,
-        });
-        for node in branch {
-            let skip = match show {
-                git_stack::config::Format::Silent => true,
-                git_stack::config::Format::Commits => false,
-                git_stack::config::Format::BranchCommits => {
-                    let protected = node.action.is_protected() || node.action.is_rebase();
-                    protected && node.branches.is_empty()
+        },
+        weight,
+        stacks,
+    };
+
+    tree
+}
+
+struct Tree<'r> {
+    root: RenderNode<'r>,
+    stacks: Vec<Vec<Self>>,
+    weight: Weight,
+}
+
+impl<'r> Tree<'r> {
+    fn skip<F>(&mut self, is_skipped: F)
+    where
+        F: Fn(&Self) -> bool + Copy,
+    {
+        for stack in self.stacks.iter_mut() {
+            let mut skipped = Vec::new();
+            for (index, child) in stack.iter_mut().enumerate() {
+                if is_skipped(child) {
+                    skipped.push(index)
+                } else {
+                    child.skip(is_skipped);
                 }
-                git_stack::config::Format::Branches => {
-                    let boring_commit = node.branches.is_empty() && node.stacks.is_empty();
-                    boring_commit
-                }
-                git_stack::config::Format::Debug => false,
-            };
-            if skip {
-                log::trace!("Skipping commit {}", node.local_commit.id);
-                continue;
             }
-            let mut child_tree = treeline::Tree::root(RenderNode {
-                repo,
-                node: Some(node),
-                palette,
-            });
-            to_tree(repo, node.stacks.as_slice(), &mut child_tree, palette, show);
-            branch_root.push(child_tree);
+            skipped.reverse();
+            for index in skipped {
+                stack.remove(index);
+            }
         }
-        tree.push(branch_root);
+    }
+
+    fn sort(&mut self) {
+        self.stacks.sort_by_key(|s| s[0].weight);
+        for stack in self.stacks.iter_mut() {
+            for child in stack.iter_mut() {
+                child.sort();
+            }
+        }
+    }
+
+    fn linearize(&mut self) {
+        self.stacks.sort_by_key(|s| s[0].weight);
+        for stack in self.stacks.iter_mut() {
+            for child in stack.iter_mut() {
+                child.linearize();
+            }
+            let append = {
+                let last = stack.last_mut().expect("stack always has at least 1");
+                if last.stacks.is_empty() {
+                    None
+                } else {
+                    last.stacks.pop()
+                }
+            };
+            stack.extend(append.into_iter().flatten());
+        }
+    }
+
+    fn into_display(self) -> termtree::Tree<RenderNode<'r>> {
+        let mut tree = termtree::Tree::root(self.root);
+        if self.stacks.len() == 1 {
+            for stack in self.stacks.into_iter() {
+                for child in stack.into_iter() {
+                    tree.push(child.into_display());
+                }
+            }
+        } else {
+            for stack in self.stacks.into_iter() {
+                let mut stack_tree = termtree::Tree::root(self.root.joint());
+                for child in stack.into_iter() {
+                    stack_tree.push(child.into_display());
+                }
+                tree.push(stack_tree);
+            }
+        }
+        tree
     }
 }
 
-struct RenderNode<'r, 'n, 'p> {
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
+enum Weight {
+    Commit(usize),
+    Head(usize),
+    Protected(usize),
+}
+
+impl Weight {
+    fn max(self, other: Self) -> Self {
+        match (self, other) {
+            (Self::Protected(s), Self::Protected(o)) => Self::Protected(s.max(o)),
+            (Self::Protected(s), _) => Self::Protected(s),
+            (_, Self::Protected(o)) => Self::Protected(o),
+            (Self::Head(s), Self::Head(o)) => Self::Head(s.max(o)),
+            (Self::Head(s), _) => Self::Head(s),
+            (_, Self::Head(s)) => Self::Head(s),
+            (Self::Commit(s), Self::Commit(o)) => Self::Commit(s.max(o)),
+        }
+    }
+}
+
+#[derive(Copy, Clone)]
+struct RenderNode<'r> {
     repo: &'r git_stack::git::GitRepo,
-    node: Option<&'n git_stack::graph::Node>,
-    palette: &'p Palette,
+    head_branch: &'r git_stack::git::Branch,
+    protected_branches: &'r git_stack::git::Branches,
+    node: Option<&'r git_stack::graph::Node>,
+    palette: &'r Palette,
+}
+
+impl<'r> RenderNode<'r> {
+    fn joint(&self) -> Self {
+        Self {
+            repo: self.repo,
+            head_branch: self.head_branch,
+            protected_branches: self.protected_branches,
+            node: None,
+            palette: self.palette,
+        }
+    }
 }
 
 // Shared implementation doesn't mean shared requirements, we want to track according to
 // requirements
 #[allow(clippy::if_same_then_else)]
-impl<'r, 'n, 'p> std::fmt::Display for RenderNode<'r, 'n, 'p> {
+impl<'r> std::fmt::Display for RenderNode<'r> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
         if let Some(node) = self.node.as_ref() {
             if node.branches.is_empty() {
@@ -845,23 +1019,39 @@ impl<'r, 'n, 'p> std::fmt::Display for RenderNode<'r, 'n, 'p> {
                     .unwrap()
                     .short_id()
                     .unwrap();
-                let style = if node.stacks.is_empty() {
+                let style = if self.head_branch.id == node.local_commit.id {
+                    self.palette.highlight
+                } else if node.action.is_protected() {
                     self.palette.info
-                } else {
+                } else if 1 < node.children.len() {
                     // Branches should be off of other branches
                     self.palette.warn
+                } else {
+                    self.palette.hint
                 };
                 write!(f, "{}", style.paint(abbrev_id.as_str().unwrap()))?;
             } else {
+                let mut branches: Vec<_> = node.branches.iter().collect();
+                branches.sort_by_key(|b| {
+                    let is_head = self.head_branch.id == b.id && self.head_branch.name == b.name;
+                    let head_first = !is_head;
+                    (head_first, &b.name)
+                });
                 write!(
                     f,
                     "{}",
-                    node.branches
-                        .iter()
+                    branches
+                        .into_iter()
                         .map(|b| {
                             format!(
                                 "{}{}",
-                                format_branch_name(b, node, self.palette),
+                                format_branch_name(
+                                    b,
+                                    node,
+                                    self.head_branch,
+                                    self.protected_branches,
+                                    self.palette
+                                ),
                                 format_branch_status(b, self.repo, node, self.palette),
                             )
                         })
@@ -876,7 +1066,7 @@ impl<'r, 'n, 'p> std::fmt::Display for RenderNode<'r, 'n, 'p> {
             )?;
 
             let summary = String::from_utf8_lossy(&node.local_commit.summary);
-            if node.action.is_protected() || node.action.is_rebase() {
+            if node.action.is_protected() {
                 write!(f, "{}", self.palette.hint.paint(summary))?;
             } else if node.local_commit.fixup_summary().is_some() {
                 // Needs to be squashed
@@ -885,7 +1075,7 @@ impl<'r, 'n, 'p> std::fmt::Display for RenderNode<'r, 'n, 'p> {
                 // Not for pushing implicitly
                 write!(f, "{}", self.palette.error.paint(summary))?;
             } else {
-                write!(f, "{}", self.palette.hint.paint(summary))?;
+                write!(f, "{}", summary)?;
             }
         } else {
             write!(f, "o")?;
@@ -897,12 +1087,22 @@ impl<'r, 'n, 'p> std::fmt::Display for RenderNode<'r, 'n, 'p> {
 fn format_branch_name<'d>(
     branch: &'d git_stack::git::Branch,
     node: &'d git_stack::graph::Node,
+    head_branch: &'d git_stack::git::Branch,
+    protected_branches: &'d git_stack::git::Branches,
     palette: &'d Palette,
 ) -> impl std::fmt::Display + 'd {
-    if node.action.is_protected() || node.action.is_rebase() {
-        palette.info.paint(branch.name.as_str())
+    if head_branch.id == branch.id && head_branch.name == branch.name {
+        palette.highlight.paint(branch.name.as_str())
     } else {
-        palette.good.paint(branch.name.as_str())
+        let protected = protected_branches.get(branch.id);
+        if protected.into_iter().flatten().contains(&branch) {
+            palette.info.paint(branch.name.as_str())
+        } else if node.action.is_protected() {
+            // Either haven't started dev or it got merged
+            palette.warn.paint(branch.name.as_str())
+        } else {
+            palette.good.paint(branch.name.as_str())
+        }
     }
 }
 
@@ -913,7 +1113,7 @@ fn format_branch_status<'d>(
     palette: &'d Palette,
 ) -> String {
     // See format_commit_status
-    if node.action.is_protected() || node.action.is_rebase() {
+    if node.action.is_protected() {
         match commit_relation(repo, branch.id, branch.pull_id) {
             Some((0, 0)) => format!(""),
             Some((local, 0)) => {
@@ -947,30 +1147,30 @@ fn format_branch_status<'d>(
         if node.branches.is_empty() {
             format!("")
         } else {
-            if node.pushable {
-                format!(" {}", palette.info.paint("(ready) "))
-            } else {
-                let branch = &node.branches[0];
-                match commit_relation(repo, branch.id, branch.push_id) {
-                    Some((0, 0)) => {
-                        format!(" {}", palette.good.paint("(pushed)"))
-                    }
-                    Some((local, 0)) => {
-                        format!(" {}", palette.info.paint(format!("({} ahead)", local)))
-                    }
-                    Some((0, remote)) => {
-                        format!(" {}", palette.warn.paint(format!("({} behind)", remote)))
-                    }
-                    Some((local, remote)) => {
-                        format!(
-                            " {}",
-                            palette
-                                .warn
-                                .paint(format!("({} ahead, {} behind)", local, remote)),
-                        )
-                    }
-                    None => {
-                        format!(" {}", palette.info.paint("(no remote)"))
+            let branch = &node.branches[0];
+            match commit_relation(repo, branch.id, branch.push_id) {
+                Some((0, 0)) => {
+                    format!(" {}", palette.good.paint("(pushed)"))
+                }
+                Some((local, 0)) => {
+                    format!(" {}", palette.info.paint(format!("({} ahead)", local)))
+                }
+                Some((0, remote)) => {
+                    format!(" {}", palette.warn.paint(format!("({} behind)", remote)))
+                }
+                Some((local, remote)) => {
+                    format!(
+                        " {}",
+                        palette
+                            .warn
+                            .paint(format!("({} ahead, {} behind)", local, remote)),
+                    )
+                }
+                None => {
+                    if node.pushable {
+                        format!(" {}", palette.info.paint("(ready)"))
+                    } else {
+                        format!("")
                     }
                 }
             }
@@ -984,7 +1184,7 @@ fn format_commit_status<'d>(
     palette: &'d Palette,
 ) -> String {
     // See format_commit_status
-    if node.action.is_protected() || node.action.is_rebase() {
+    if node.action.is_protected() {
         format!("")
     } else if node.action.is_delete() {
         format!("{} ", palette.warn.paint("(drop)"))
@@ -1006,6 +1206,10 @@ fn commit_relation(
     remote: Option<git2::Oid>,
 ) -> Option<(usize, usize)> {
     let remote = remote?;
+    if local == remote {
+        return Some((0, 0));
+    }
+
     let base = repo.merge_base(local, remote)?;
     let local_count = repo
         .commits_from(local)
@@ -1024,17 +1228,19 @@ struct Palette {
     warn: yansi::Style,
     info: yansi::Style,
     good: yansi::Style,
+    highlight: yansi::Style,
     hint: yansi::Style,
 }
 
 impl Palette {
     pub fn colored() -> Self {
         Self {
-            error: yansi::Style::new(yansi::Color::Red),
-            warn: yansi::Style::new(yansi::Color::Yellow),
-            info: yansi::Style::new(yansi::Color::Blue),
-            good: yansi::Style::new(yansi::Color::Green),
-            hint: yansi::Style::new(yansi::Color::Blue).dimmed(),
+            error: yansi::Style::new(yansi::Color::Red).bold(),
+            warn: yansi::Style::new(yansi::Color::Yellow).bold(),
+            info: yansi::Style::new(yansi::Color::Blue).bold(),
+            good: yansi::Style::new(yansi::Color::Cyan).bold(),
+            highlight: yansi::Style::new(yansi::Color::Green).bold(),
+            hint: yansi::Style::new(yansi::Color::Unset).dimmed(),
         }
     }
 
@@ -1044,6 +1250,7 @@ impl Palette {
             warn: yansi::Style::default(),
             info: yansi::Style::default(),
             good: yansi::Style::default(),
+            highlight: yansi::Style::default(),
             hint: yansi::Style::default(),
         }
     }
