@@ -1,8 +1,7 @@
-#[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum Action {
     Pick,
     Protected,
-    Rebase(git2::Oid),
     Delete,
 }
 
@@ -15,10 +14,6 @@ impl Action {
         matches!(self, Action::Protected)
     }
 
-    pub fn is_rebase(&self) -> bool {
-        matches!(self, Action::Rebase(_))
-    }
-
     pub fn is_delete(&self) -> bool {
         matches!(self, Action::Delete)
     }
